# Overview
This folder contains all the Webserver and UI code. The webserver is implemented using Flask at the backend with html and js at the front end. The Webserver structure is as follows:
```
physioai 
    ├── app.py - Flask routing file
    ├── static
    │   ├── default.css - style sheet
    │   ├── images - logos and other images
    │   ├── js   - Holds java scripts
    │   ├── output - Output folder, synced to AWS physioai-outvideo bucket
    ├── templates - HTML pages
```

# Running locally on a PC
A development server can be run locally on a PC to test
1. Install `flask` and any dependencies
2. Run `python app.py`
3. Typically a local webserver would start at the following address: http://127.0.0.1:5000
4. Paste the above in browser

# Installation instructions on AWS
EC2 instance t2-micro is sufficient for the webserver. Ubuntu is used for OS. Installation steps:
1.Setup two s3 buckets (if not already done) as follows:
  - physioai-invideo - the videos captured by the capture.html page will be uploaded here.
  - physioai-outvideo - the output of the model running on SageMaker will be in this folder. 
2. Follow the instructions in this youtube video to install flask backend: https://youtu.be/z5XiVh6v4uI?si=dMJrjmIvoVhuK6DD . Replace the app.py in the example with the app.py in this folder
   - Above steps create a virtual environment and install necessary packages including flask. A `requirements.txt` file is provided in the webserver folder for reference
   - A service called `physioai.service` is created following the above video and installed in /etc/systemd/system/ on aws. Sample service file is provided in this folder
   - nginx server configuration needs to be updated to increase the maximum size of the upload video file. Edit `/etc/nginx/nginx.conf` to add the following line `client_max_body_size 50M;` under `Basic Settings`
   - Enable and start the `physioai.service` using `systemctl` commands. `sudo` may need to be used to run `systemctl` command
   - For security, create self signed certificates or use a CA signed certificates. Place the certificate `/etc/ssl/certs` and private key in `/etc/ssl/private` folders on the EC2 instance. Edit `/etc/nginx/sites-enabled/default` to add  the following in the server section
     
     ```
      ssl_certificate /etc/ssl/certs/nginx-self-signed.crt;
      ssl_certificate_key /etc/ssl/private/nginx-self-signed.key;
     ```
4. Setup a sync service between aws s3 bucket physioai-outvideo and static/output folder using `sync_s3.sh` file. A cronjob can be setup to periodically sync.


