from flask import Flask, render_template, request, redirect, url_for, session, flash
import os
from functools import wraps  # Import wraps
import boto3
from werkzeug.utils import secure_filename
import subprocess
import datetime
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

logger.info('starting flask app')
app = Flask(__name__)
s3 = boto3.client('s3')
app.secret_key = 'your_secret_key'

def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        if 'logged_in' not in session:
            flash('You need to be logged in to access this page.')
            return redirect(url_for('login'))
        return f(*args, **kwargs)
    return decorated_function

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/login', methods=['GET', 'POST'])
def login():
    if request.method == 'POST':
        username = request.form['username']
        password = request.form['password']
        if username == 'demo' and password == 'physioai':
            session['logged_in'] = True
            return redirect(url_for('practitioner'))
        else:
            error = "Invalid username or password"
            return render_template('login.html', error=error)
    return render_template('login.html')

@app.route('/logout')
def logout():
    session.pop('logged_in', None)
    return redirect(url_for('login'))

@app.route('/practitioner')
@login_required
def practitioner():
    return render_template('practitioner.html')

@app.route('/client')
@login_required
def client():
    return render_template('client.html')

@app.route('/capture')
@login_required
def capture():
    return render_template('capture.html')

@app.route('/about')
def about():
    return render_template('about.html')

@app.route('/upload', methods=['POST'])
def upload_video():
    logger.info('uplading captured video')
    video_data = request.files['video']
    video_name = request.form.get('videoname', 'captured_video')
    filename = secure_filename(video_name + '.mp4')
    #logger.info(str(video_name), str(filename))

    # Save the file temporarily on the server
    temp_path = '/tmp/' + filename
    video_data.save(temp_path)

    current_time = datetime.datetime.now()
    #timestamp = current_time.strftime("%m%d%y%H%M%S")
    #timestamp = current_time.strftime("%y-%m-%d %H-%M-%S")
    timestamp = current_time.strftime("%Y-%m-%d") + " " + current_time.strftime("%H-%M-%S")
    #out_filename = secure_filename(video_name + "_" +  timestamp + ".mp4")
    out_filename = video_name + "_" +  timestamp + ".mp4"
    temp_out = '/tmp/' + out_filename

    #command = ["ffmpeg", "-i", temp_path, "-c:v", "copy", "-c:a", "aac", "-strict", "experimental", temp_out]
    command = ['ffmpeg', '-i', temp_path, '-c:v', 'copy', '-c:a', 'aac', '-strict', 'experimental', temp_out]
    #command = ['ffmpeg', '-i', temp_path, '-c:v', 'libx264', '-c:a', 'aac', temp_out]
    subprocess.run(command, check=True)

    #command = ["cp", temp_out, temp_path]
    #subprocess.run(command, check=True)

    # Upload to S3
    logger.info('Uploading file to s3')
    bucket_name = 'physioai-invideo'  # Replace with your bucket name
    s3.upload_file(temp_out, bucket_name, out_filename)
    logger.info('upload complete')

    # Optionally, remove the temporary file if you want
    os.remove(temp_path)
    os.remove(temp_out)
    logger.info('removed loca files')

    return redirect(url_for('index'))



if __name__ == '__main__':
    app.run(debug=True)

