let video = document.getElementById('video');
let mediaRecorder;
let startRecord = document.getElementById('startRecord');
let stopRecord = document.getElementById('stopRecord');

const countdownDuration =  15;
const recordingTime = 30;

// Get the overlay element to display the countdown
const countdownOverlay = document.querySelector('.countdown-overlay');

const constraints = {
    video: {
        width: { ideal: 720}, // Desired width
        height: { ideal: 1080}, // Desired height
        facingMode: 'user' // 'user' for front camera, 'environment' for back camera
    }
};

// Request access to the media devices
navigator.mediaDevices.getUserMedia({ video: true })
//navigator.mediaDevices.getUserMedia(constraints)
//navigator.mediaDevices.getUserMedia({ video: {facingMode: "user"}})
    .then(stream => {
	video.setAttribute('autoplay','');
	video.setAttribute('muted','');
	video.setAttribute('playsinline','');
        video.srcObject = stream;
        mediaRecorder = new MediaRecorder(stream);

        // Event listener for the "Start Recording" button
        startRecord.addEventListener('click', () => {
	    countdownOverlay.style.display = 'flex';
            // Start the countdown
            let counter = countdownDuration;
            countdownOverlay.textContent = counter; // Display the initial counter value
            const countdownInterval = setInterval(() => {
                counter--;
                countdownOverlay.textContent = counter; // Update the counter display
                if (counter <= 0) {
                    clearInterval(countdownInterval); // Stop the countdown
		    countdownOverlay.style.display = 'none';
                    mediaRecorder.start(); // Start recording
                    startRecord.style.display = 'none';
                    stopRecord.style.display = 'block';

                    // Set a timeout to stop the recording after the specified recording time
                    setTimeout(() => {
                        mediaRecorder.stop();
                        startRecord.style.display = 'block';
                        stopRecord.style.display = 'none';
                    }, recordingTime * 1000);
                }
            }, 1000);
        });


        // Event listener for the "Stop Recording" button
        stopRecord.addEventListener('click', () => {
            mediaRecorder.stop();
	    startRecord.style.display = 'block';
	    stopRecord.style.display = 'none';
        });

        // Handle data available after recording stops
        mediaRecorder.ondataavailable = async (event) => {
            const blob = event.data;
            const formData = new FormData();
            let videoNameInput = document.getElementById('videoName');
	    let name = videoNameInput.value; 
	    //let videoName = videoNameInput.value.trim() || videoNameInput.placeholder;
            if (name === '') {
		    name = 'John';
            }
            formData.append('video', blob);
	    formData.append('videoname', name);

	    countdownOverlay.style.display = 'flex';
            countdownOverlay.textContent = 'Done';
            try {
                const response = await fetch('/upload', {
                    method: 'POST',
                    body: formData,
                });

                if (response.ok) {
                    console.log('Video uploaded successfully!');
                } else {
                    console.error('Error uploading video.');
                }
            } catch (error) {
                console.error('Network error:', error);
            }
        };
    })
    .catch(error => {
        console.error('Error accessing media devices:', error);
    });

;

