const chartContainer = d3.select("#chart-container");
const containerWidth = parseFloat(chartContainer.style("width"));
const containerHeight = parseFloat(chartContainer.style("height"));
const chartContainer2 = d3.select("#chart-container2");
const containerWidth2 = parseFloat(chartContainer2.style("width"));
const containerHeight2 = parseFloat(chartContainer2.style("height"));
const chartContainer3 = d3.select("#chart-container3");
const containerWidth3 = parseFloat(chartContainer3.style("width"));
const containerHeight3 = parseFloat(chartContainer3.style("height"));

// Cotainer 1 - donut charts for some key numbers
// Total number of videos
// Total number of reps
// Videos uploaded in last 24 hrs - simulating pending review by practitioner
// Total percentage of correct reps across all videos and keypoints

const ringWidth = containerWidth/4; // four rings hence divide container into 4 
const ringHeight = containerHeight;
const outer_radius = Math.min(ringWidth, ringHeight)/2  - 10;
const inner_radius = outer_radius - 10;

const color = d3.scaleOrdinal(d3.schemeCategory10)

const appendSvg = (container, width, height) => {
    return container
        .append("svg")
        .attr("width", width)
        .attr("height", height)
        .append("g")
        .attr("transform", `translate(${width / 2}, ${height / 2})`);
};

const pie = d3.pie()
    .value(d => Math.max(1, d))

const arc = d3.arc()
    .innerRadius(inner_radius-30)
    .outerRadius(outer_radius-10); // Adjust outer radius as needed

const svgVideos = appendSvg(chartContainer, ringWidth, ringHeight);
const svgReps = appendSvg(chartContainer, ringWidth, ringHeight);
const svgPendingReview = appendSvg(chartContainer, ringWidth, ringHeight);
const svgCorrectness = appendSvg(chartContainer, ringWidth, ringHeight);

// Container 2 
const margin = {top: 20, right: 10, bottom: 50, left: 30};
const histWidth = containerWidth2 / 2 - margin.left - margin.right;
const histHeight = containerHeight2 - margin.top - margin.bottom;

const barChartWidth = containerWidth2 / 2 - margin.left - margin.right; 
const barChartHeight = containerHeight2 - margin.top - margin.bottom;

const svgHist = chartContainer2 //d3.select("#histogram")
    .append("svg")
    .attr("width", containerWidth2 / 2)
    .attr("height", containerHeight2)
    .append("g")
    //.attr("transform", `translate(${mrgin.left},${margin.top})`);
    .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

const svgBarChart = chartContainer2
        .append("svg")
        .attr("width", containerWidth2 /2 )
        .attr("height", containerHeight2)
        .append("g")
        //attr("transform", `translate(${margin.left},${margin.top})`)
        .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

// Container 3
const lineMargin = { top: 60, right: 30, bottom: 50, left: 30 };
const lineWidth = containerWidth3 - lineMargin.left - lineMargin.right;
const lineHeight = containerHeight3 - lineMargin.top - lineMargin.bottom;

// Function to draw a ring
const drawRing = (svg, total, label,colors) => {
    // Draw the outer circle
    svg.selectAll('path')
    .data(pie(total))
    .enter()
    .append('path')
    .attr('d', arc)
    //.attr('fill', (d, i) =>  color(i));
    .attr('fill', function(d,i) {
        return colors[i % colors.length];
    });

    // Add the text inside the ring
    svg.append("text")
    .attr("text-anchor", "middle")
    .attr("dy", "0.3em")
    .attr("font-size", "24px")
    .attr("fill", "#333")
    .text(total[0]);

    // Add the label below the number
    svg.append("text")
    .attr("text-anchor", "middle")
    .attr("dy", "2.5em")
    .attr("font-size", "16px")
    .attr("fill", "#333")
    .text(label);
};

// Load the CSV data
d3.csv("static/output/practitioner_dashboard.csv").then(data => {

    // Process container1 - compute variables & draw rings
    // count videos by keying in on date, name and version from csv
    var uniqueVideos = d3.rollup(data, 
                             v => v.length, 
                             d => `${d.Date} - ${d.Name} - ${d.Version}`);
    var totalVideos = uniqueVideos.size;

    // Filter data for a specific key point to count repetitions
    const keyPoint = "left_hip"; // You can change this to any key point
    const filteredData = data.filter(d => d.Key_point === keyPoint);
    const totalReps = filteredData.length;

    // Filter data for videos in the last 24 hours
    const now = new Date();
    const oneDayAgo = new Date(now - 24 * 60 * 60 * 1000);
    const recentVideos = new Map();
    data.forEach(d => {
        const videoDate = new Date(d.Date);
        const videoId = `${d.Date}_${d.Name}_${d.Version}`;
        if (videoDate >= oneDayAgo && !recentVideos.has(videoId)) {
            recentVideos.set(videoId, 1);
        }
    });
    
    const pendingReview = recentVideos.size;

    // Calculate the correctness percentage - count all rows and sum correctness
    const correctnessPercentage = (d3.mean(data, d => +d.Correctness) * 100).toFixed(1);

    
    // Draw the rings with different colors
    drawRing(svgVideos, [totalVideos], "Videos", ["#1f78b4"]);
    drawRing(svgReps, [totalReps], "Reps", ["#1f78b4"]);
    drawRing(svgPendingReview, [pendingReview], ["Pending"],
             pendingReview === 0 ? ["#1f78b4"] : ["#d6604d"]);
    drawRing(svgCorrectness, [correctnessPercentage, 100-correctnessPercentage], 
                           "%Correctness", ["#66c2a5", "#fc8d62"]);

    // container 2 processing 
    // Process data for the stacked histogram
    const repetitionGroups = d3.rollup(data,
        v => ({
            correct: v.filter(d => d.Correctness == 1).length,
            incorrect: v.filter(d => d.Correctness == 0).length
        }),
        d => d.Repetition
    );

    const stackedData = Array.from(repetitionGroups, ([key, value]) => ({
        Repetition: key,
        Correct: value.correct,
        Incorrect: value.incorrect
    }));

    const stack = d3.stack()
        .keys(["Correct", "Incorrect"])
        .order(d3.stackOrderNone)
        .offset(d3.stackOffsetNone);

    const layers = stack(stackedData);

    const xHist = d3.scaleBand()
        .domain(stackedData.map(d => d.Repetition))
        .range([0, histWidth])
        .padding(0.1);

    const yHist = d3.scaleLinear()
        .domain([0, d3.max(stackedData, d => d.Correct + d.Incorrect)])
        .nice()
        .range([histHeight, 0]);

    const z = d3.scaleOrdinal()
        .domain(["Correct", "Incorrect"])
        .range(["#66c2a5", "#fc8d62"]);

    svgHist.append("g")
        .attr("class", "x axis")
        .attr("transform", `translate(0,${histHeight})`)
        .call(d3.axisBottom(xHist))
        .append("text")
        .attr("x", histWidth / 2)
        .attr("y", margin.bottom - 10)
        .attr("fill", "currentColor")
        .attr("text-anchor", "middle")
        .text("Repetitions");

    svgHist.append("g")
        .attr("class", "y axis")
        .call(d3.axisLeft(yHist))
        .call(g => g.append("text")
            .attr("transform", "rotate(-90)")
            .attr("x", -margin.top)
            .attr("y", -margin.left + 5)
            .attr("fill", "currentColor")
            .attr("text-anchor", "end")
            .text(""));

    const layer = svgHist.selectAll(".layer")
        .data(layers)
        .enter().append("g")
        .attr("class", "layer")
        .attr("fill", d => z(d.key));

    layer.selectAll("rect")
        .data(d => d)
        .enter().append("rect")
        .attr("x", d => xHist(d.data.Repetition))
        .attr("y", d => yHist(d[1]))
        .attr("height", d => yHist(d[0]) - yHist(d[1]))
        .attr("width", xHist.bandwidth());

    svgHist.append("text")
        .attr("x", histWidth / 2)
        .attr("y", 0)
        .attr("text-anchor", "middle")
        .style("font-size", "18px")
        .text("Repetition Summary (All Measures)");

    // Create the bar chart
    const keys = Array.from(new Set(data.map(d => d.Key_point)));
    const correctData = keys.map(key => {
        return {
            key_point: key,
            correctness: data.filter(d => d.Key_point === key && d.Correctness === "1").length
        };
    });
    const incorrectData = keys.map(key => {
        return {
            key_point: key,
            correctness: data.filter(d => d.Key_point === key && d.Correctness === "0").length
        };
    });

    const maxCorrect = d3.max(correctData, d => d.correctness);
    const maxIncorrect = d3.max(incorrectData, d => d.correctness);
    const maxY = Math.max(maxCorrect, maxIncorrect);

    const x = d3.scaleBand()
        .domain(keys)
        .range([0, barChartWidth])
        .padding(0.1);

    const y = d3.scaleLinear()
        .domain([0, maxY])
        .nice()
        .range([barChartHeight, 0]);

    //x.domain(keys);
    //y.domain([0, maxY]);
    //const width = barChartWidth - margin.left - margin.right;
    //const height = barChartHeight - margin.top - margin.bottom;

    svgBarChart.append("g")
        .attr("class", "x axis")
        .attr("transform", `translate(0,${barChartHeight})`)
        .call(d3.axisBottom(x));

    svgBarChart.append("g")
        .attr("class", "y axis")
        .call(d3.axisLeft(y));

    svgBarChart.append("text")
        .attr("x", barChartWidth / 2)
        .attr("y", 0) // Adjust position based on top margin
        .attr("text-anchor", "middle")
        .style("font-size", "18px")
        .text("Key Point Summary"); // Title text

    const barWidth = x.bandwidth() / 2;

    svgBarChart.selectAll(".bar.correct")
        .data(correctData)
        .enter().append("rect")
        .attr("class", "bar correct")
        .attr("x", d => x(d.key_point))
        .attr("width", barWidth)
        .attr("y", d => y(d.correctness))
        .attr("height", d => barChartHeight - y(d.correctness))
        .attr("fill", "#66c2a5");

    svgBarChart.selectAll(".bar.incorrect")
        .data(incorrectData)
        .enter().append("rect")
        .attr("class", "bar incorrect")
        .attr("x", d => x(d.key_point) + barWidth)
        .attr("width", barWidth)
        .attr("y", d => y(d.correctness))
        .attr("height", d => barChartHeight - y(d.correctness))
        .attr("fill", "#fc8d62");

    // Add legend
    const legend = svgBarChart.selectAll(".legend")
        .data(["Correct", "Incorrect"])
        .enter().append("g")
        .attr("class", "legend")
        .attr("transform", (d, i) => `translate(0,${i * 20})`);

    legend.append("rect")
        .attr("x", barChartWidth - 18)
        .attr("width", 18)
        .attr("height", 18)
        .style("fill", (d, i) => i === 0 ? "#66c2a5" : "#fc8d62");

    legend.append("text")
        .attr("x", barChartWidth - 30)
        .attr("y", 2)
        .attr("dy", ".35em")
        .style("text-anchor", "end")
        .text(d => d);

    //Container 3
    var parseDate = d3.timeParse("%Y-%m-%d");

    // Create dropdown menu
    var names = Array.from(new Set(data.map(d => d.Name)));
    console.log(names)
    var dropdown = d3.select("#filter-container")
        .append("select")
        .attr("id", "nameDropdown")
        .on("change", updateChart);

    dropdown.selectAll("option")
        .data(names)
        .enter()
        .append("option")
        .text(d => d)
        .attr("value", d => d);

    // Initial chart rendering
    updateChart();

    function updateChart() {
            var selectedName = d3.select("#nameDropdown").property("value");
            var filteredData = data.filter(d => d.Name === selectedName);

            // Remove existing chart before drawing new one
            chartContainer3.selectAll("svg").remove();
            
            var sum_data = d3.rollup(filteredData,
                                     v => d3.mean(v, leaf => leaf.Correctness)*100,
                                     d => d.Date);

            var lineData = Array.from(sum_data, ([date, sum]) => ({date: d3.timeParse("%Y-%m-%d")(date), sum}));
            lineData.sort((a, b) => a.date - b.date);
            
            var dateExtent = d3.extent(lineData, d => d.date);

            var paddingDays = (dateExtent[1] - dateExtent[0]) / (1000 * 60 * 60 * 24) * 0.1; // 10% padding
            var adjustedStartDate = d3.timeDay.offset(dateExtent[0], -Math.ceil(paddingDays));
            var adjustedEndDate = d3.timeDay.offset(dateExtent[1], Math.ceil(paddingDays));

            var dateRange = d3.timeDays(adjustedStartDate, adjustedEndDate);
            var completeData = dateRange.map(date => {
                var found = lineData.find(d => d.date.getTime() === date.getTime());
                return found ? found : {date: date, sum: 0};
            });

            const svgLine = chartContainer3
                 .append("svg")
                 .attr("width", containerWidth3)
                 .attr("height", containerHeight3)
                 .append("g")
                 .attr("transform", "translate(" + lineMargin.left + "," + lineMargin.top + ")");

            var xScale = d3.scaleTime()
                .domain([adjustedStartDate, adjustedEndDate])
                .range([0, lineWidth]);

            var yScale = d3.scaleLinear()
                //.domain([0, d3.max(lineData, function(d) { return d.sum; })])
                .domain([0, 100])
                .range([lineHeight,  0]);

            var line = d3.line()
                .x(function(d) { return xScale(d.date); })
                .y(function(d) { return yScale(d.sum); });

            svgLine.append("path")
                .datum(lineData)
                .attr("class", "line")
                .attr("fill", "none")
                .attr("stroke", "#1f78b4")
                .attr("stroke-width", 4)
                .attr("d", line);

            svgLine.append("g")
                .attr("transform", "translate(0," + lineHeight + ")")
                .call(d3.axisBottom(xScale));

            svgLine.append("g")
                .call(d3.axisLeft(yScale));

            svgLine.selectAll(".dot")
                .data(lineData)
                .enter()
                .append("circle") // append circle elements
                .attr("class", "dot") // apply a class for styling
                .attr("cx", function(d) { return xScale(d.date) })
                .attr("cy", function(d) { return yScale(d.sum) })
                .attr("r", function(d) { return d.sum> 0 ? 5 : 0 }) // show dot if sum greater than 0
                .style("fill", function(d) { return d.sum > 70? "#66c2a5" : "#fc8d62"; });

            // Add y-axis label
            svgLine.append("text")
                .attr("class", "axis-label")
                .attr("transform", "rotate(-90)")
                .attr("y", 0 - margin.left + 30)
                .attr("x", 0 - (lineHeight/ 2))
                .attr("dy", "1em")
                .style("text-anchor", "middle")
                .text("Correctness %");

            // Add title
            svgLine.append("text")
                .attr("class", "chart-title")
                .attr("x", lineWidth/ 2)
                .attr("y", 0 - (margin.top / 2))
                .style("text-anchor", "middle")
                .text("Correctness Over Time for " + selectedName);

     }
}).catch(error => {
    console.error('Error loading or parsing data:', error);
});

